use crate::lavina_clients::errors::*;
use reqwest::Client;
use crate::configuration::AppConfig;

pub struct LavinaHttpClient {
    client: Client,
    create_player_url: String,
    set_password_url: String,
}

impl LavinaHttpClient {
    pub fn new(config: &AppConfig) -> Self {
        LavinaHttpClient {
            client: Client::new(),

            create_player_url: format!(
                "http://{}:{}/mgmt/create_player",
                config.local_server_address, config.lavina_http_port
            ),
            set_password_url: format!(
                "http://{}:{}/mgmt/set_password",
                config.local_server_address, config.lavina_http_port
            ),
        }
    }


    pub async fn new_player(&self, name: &str, password: &str) -> Result<(), PlayerError> {
        let mut create_query = std::collections::HashMap::new();
        create_query.insert("name", name);

        let create_user_response = self.client.post(&self.create_player_url)
            .json(&create_query).send().await.map_err(PlayerError::from)?;

        let response_message = create_user_response.text().await.map_err(PlayerError::from)?;

        if response_message.contains("error returned from database: (code: 2067) UNIQUE constraint failed: users.name") {
            log::info!("User {:?} already exists. Response from server: {:?}", name, response_message);
            return Err(PlayerError::UserAlreadyExists(name.to_string()));
        } else {
            log::info!("Created user. Response: {:?}", response_message);
            self.set_password(name, password).await?;
            return Ok(());
        }
    }

    pub async fn set_password(&self, name: &str, password: &str) -> Result<(), PlayerError> {
        let mut set_pass_query = std::collections::HashMap::new();
        set_pass_query.insert("player_name", name);
        set_pass_query.insert("password", password);

        let set_password_response = self.client.post(&self.set_password_url)
            .json(&set_pass_query).send().await.map_err(PlayerError::from)?;

        let _ = set_password_response.text().await.map_err(PlayerError::from)?;
        log::info!("Password set for user: {:?}", name);
        Ok(())
    }
}
